precision highp float;

uniform sampler2D u_text;

varying highp vec2 textureCoordinate;

uniform float u_startTime;
uniform float u_endTime;
uniform float u_currentTime;

uniform mat4 u_textInfo;

float calcIntensity(vec2 textureCoordinate, float lineCount, float currentTime, float totalTime, float offset) {
    
    // 计算当前刷子位置
    float current = currentTime / totalTime;
    float progress = current * lineCount - offset;

    // 计算当前像素位置
    float row = floor(textureCoordinate.y * lineCount); // 当前第row行
    float currentPixel = row + textureCoordinate.x;
    
    // 比较像素与刷子距离,换算为intensity
    float delta = min(offset, max(0.0, currentPixel - progress));
    float intensity = (offset - delta) / offset;
    return intensity;
}

void main() {
    
    float current = u_currentTime - u_startTime;
    float total = u_endTime - u_startTime;

    float intensity;    
    if (current < total * 6.0 / 11.0) { // 前6/11淡入
        float lineCount = u_textInfo[3][3];
        float offset = lineCount / 2.;
        intensity = calcIntensity(textureCoordinate, u_textInfo[3][3], current, total * 6.0/ 11.0, offset);
    }
    else if (current < total * 10.0 / 11.0) { // 中间保持
        intensity = 1.0;
    }
    else { // 后1/11淡出
        intensity = 11.0 * (u_endTime - u_currentTime) / total;
    }

    float alpha = texture2D(u_text, textureCoordinate).a;
    gl_FragColor = vec4(alpha, alpha, alpha, alpha) * intensity;
}

